@@ -7,12 +7,16 @@ from binascii import crc32
 from requests.models import Request, Response
 from localstack import config
 from localstack.utils.aws import aws_stack, aws_responses
-from localstack.utils.common import to_bytes, to_str, clone
+from localstack.utils.common import to_bytes, to_str, clone, select_attributes
 from localstack.utils.analytics import event_publisher
+from localstack.utils.bootstrap import is_api_enabled
 from localstack.services.awslambda import lambda_api
 from localstack.services.generic_proxy import ProxyListener
 from localstack.services.dynamodbstreams import dynamodbstreams_api
 
+# set up logger
+LOGGER = logging.getLogger(__name__)
+
 # cache table definitions - used for testing
 TABLE_DEFINITIONS = {}
 
@@ -22,8 +26,8 @@ TABLE_TAGS = {}
 # action header prefix
 ACTION_PREFIX = 'DynamoDB_20120810'
 
-# set up logger
-LOGGER = logging.getLogger(__name__)
+# maps global table names to configurations
+GLOBAL_TABLES = {}
 
 # list of actions subject to throughput limitations
 THROTTLED_ACTIONS = [
@@ -65,7 +69,20 @@ class ProxyListenerDynamoDB(ProxyListener):
             # Check if table exists, to avoid error log output from DynamoDBLocal
             table_names = ddb_client.list_tables()['TableNames']
             if to_str(data['TableName']) in table_names:
-                return 200
+                return error_response(message='Table already created',
+                                      error_type='ResourceInUseException', code=400)
+
+        if action == '%s.CreateGlobalTable' % ACTION_PREFIX:
+            return create_global_table(data)
+
+        elif action == '%s.DescribeGlobalTable' % ACTION_PREFIX:
+            return describe_global_table(data)
+
+        elif action == '%s.ListGlobalTables' % ACTION_PREFIX:
+            return list_global_tables(data)
+
+        elif action == '%s.UpdateGlobalTable' % ACTION_PREFIX:
+            return update_global_table(data)
 
         elif action in ('%s.PutItem' % ACTION_PREFIX, '%s.UpdateItem' % ACTION_PREFIX, '%s.DeleteItem' % ACTION_PREFIX):
             # find an existing item and store it in a thread-local, so we can access it in return_response,
@@ -270,19 +287,20 @@ class ProxyListenerDynamoDB(ProxyListener):
             event_publisher.fire_event(event_publisher.EVENT_DYNAMODB_CREATE_TABLE,
                 payload={'n': event_publisher.get_hash(data['TableName'])})
 
-            if data.get('Tags'):
+            if data.get('Tags') and response.status_code == 200:
                 table_arn = json.loads(response._content)['TableDescription']['TableArn']
                 TABLE_TAGS[table_arn] = {tag['Key']: tag['Value'] for tag in data['Tags']}
 
             return
 
         elif action == '%s.DeleteTable' % ACTION_PREFIX:
+            table_arn = json.loads(response._content).get('TableDescription', {}).get('TableArn')
             event_publisher.fire_event(
                 event_publisher.EVENT_DYNAMODB_DELETE_TABLE,
                 payload={'n': event_publisher.get_hash(data['TableName'])}
             )
-
-            TABLE_TAGS.pop(json.loads(response._content).get('TableDescription', {}).get('TableArn', None), None)
+            self.delete_all_event_source_mappings(table_arn)
+            TABLE_TAGS.pop(table_arn, None)
 
             return
 
@@ -320,6 +338,10 @@ class ProxyListenerDynamoDB(ProxyListener):
             forward_to_lambda(records)
             forward_to_ddb_stream(records)
 
+    # -------------
+    # UTIL METHODS
+    # -------------
+
     def prepare_batch_write_item_records(self, record, data):
         records = []
         i = 0
@@ -401,6 +423,18 @@ class ProxyListenerDynamoDB(ProxyListener):
                 records.append(new_record)
         return records
 
+    def delete_all_event_source_mappings(self, table_arn):
+        if table_arn:
+            # fix start dynamodb service without lambda
+            if not is_api_enabled('lambda'):
+                return
+
+            lambda_client = aws_stack.connect_to_service('lambda')
+            result = lambda_client.list_event_source_mappings(EventSourceArn=table_arn)
+            for event in result['EventSourceMappings']:
+                event_source_mapping_id = event['UUID']
+                lambda_client.delete_event_source_mapping(UUID=event_source_mapping_id)
+
     @staticmethod
     def _thread_local(name, default=None):
         try:
@@ -409,6 +443,59 @@ class ProxyListenerDynamoDB(ProxyListener):
             return default
 
 
+def create_global_table(data):
+    table_name = data['GlobalTableName']
+    if table_name in GLOBAL_TABLES:
+        return get_error_message('Global Table with this name already exists', 'GlobalTableAlreadyExistsException')
+    GLOBAL_TABLES[table_name] = data
+    for group in data.get('ReplicationGroup', []):
+        group['ReplicaStatus'] = 'ACTIVE'
+        group['ReplicaStatusDescription'] = 'Replica active'
+    result = {'GlobalTableDescription': data}
+    return result
+
+
+def describe_global_table(data):
+    table_name = data['GlobalTableName']
+    details = GLOBAL_TABLES.get(table_name)
+    if not details:
+        return get_error_message('Global Table with this name does not exist', 'GlobalTableNotFoundException')
+    result = {'GlobalTableDescription': details}
+    return result
+
+
+def list_global_tables(data):
+    result = [select_attributes(tab, ['GlobalTableName', 'ReplicationGroup']) for tab in GLOBAL_TABLES.values()]
+    result = {'GlobalTables': result}
+    return result
+
+
+def update_global_table(data):
+    table_name = data['GlobalTableName']
+    details = GLOBAL_TABLES.get(table_name)
+    if not details:
+        return get_error_message('Global Table with this name does not exist', 'GlobalTableNotFoundException')
+    for update in data.get('ReplicaUpdates', []):
+        repl_group = details['ReplicationGroup']
+        # delete existing
+        delete = update.get('Delete')
+        if delete:
+            details['ReplicationGroup'] = [g for g in repl_group if g['RegionName'] != delete['RegionName']]
+        # create new
+        create = update.get('Create')
+        if create:
+            exists = [g for g in repl_group if g['RegionName'] == create['RegionName']]
+            if exists:
+                continue
+            new_group = {
+                'RegionName': create['RegionName'], 'ReplicaStatus': 'ACTIVE',
+                'ReplicaStatusDescription': 'Replica active'
+            }
+            details['ReplicationGroup'].append(new_group)
+    result = {'GlobalTableDescription': details}
+    return result
+
+
 def find_existing_item(put_item, table_name=None):
     table_name = table_name or put_item['TableName']
     ddb_client = aws_stack.connect_to_service('dynamodb')
@@ -431,6 +518,8 @@ def find_existing_item(put_item, table_name=None):
 
     req = {'TableName': table_name, 'Key': search_key}
     existing_item = aws_stack.dynamodb_get_item_raw(req)
+    if not existing_item:
+        return existing_item
     if 'Item' not in existing_item:
         if 'message' in existing_item:
             table_names = ddb_client.list_tables()['TableNames']
@@ -441,13 +530,17 @@ def find_existing_item(put_item, table_name=None):
     return existing_item.get('Item')
 
 
-def get_table_not_found_error():
-    response = error_response(message='Cannot do operations on a non-existent table',
-                              error_type='ResourceNotFoundException')
+def get_error_message(message, error_type):
+    response = error_response(message=message, error_type=error_type)
     fix_headers_for_updated_response(response)
     return response
 
 
+def get_table_not_found_error():
+    return get_error_message(message='Cannot do operations on a non-existent table',
+                             error_type='ResourceNotFoundException')
+
+
 def fix_headers_for_updated_response(response):
     response.headers['content-length'] = len(to_bytes(response.content))
     response.headers['x-amz-crc32'] = calculate_crc32(response)
@@ -489,7 +582,11 @@ def forward_to_lambda(records):
             'Records': [record]
         }
         for src in sources:
-            lambda_api.run_lambda(event=event, context={}, func_arn=src['FunctionArn'])
+            if src.get('State') != 'Enabled':
+                continue
+
+            lambda_api.run_lambda(event=event, context={}, func_arn=src['FunctionArn'],
+                asynchronous=not config.SYNCHRONOUS_DYNAMODB_EVENTS)
 
 
 def forward_to_ddb_stream(records):
