@@ -391,6 +391,7 @@ def start_infra_in_docker():
 
     # get port ranges defined via DOCKER_FLAGS (if any)
     user_flags = extract_port_flags(user_flags, port_mappings)
+    plugin_run_params = extract_port_flags(plugin_run_params, port_mappings)
 
     # construct default port mappings
     if service_ports.get('edge') == 0:
@@ -399,11 +400,6 @@ def start_infra_in_docker():
     for port in service_ports.values():
         port_mappings.add(port)
 
-    if services:
-        port_mappings = PortMappings()
-        for port in set(service_ports.values()):
-            port_mappings.add(port)
-
     env_str = ''
     for env_var in config.CONFIG_ENV_VARS:
         value = os.environ.get(env_var, None)
